@@ -61,6 +61,7 @@ import functools
 import logging
 
 from keystoneauth1 import exceptions as ksa_exc
+from keystoneauth1.loading import adapter as ksa_adapter
 from keystoneauth1.loading import base as ksa_base
 from keystoneauth1.loading import session as ksa_session
 from keystoneclient.v3 import client as ks_client
@@ -72,12 +73,9 @@ from pycadf.helper import api
 from pycadf import measurement as cadf_measurement
 from pycadf import metric as cadf_metric
 from pycadf import resource as cadf_resource
-import six
-import six.moves.queue as queue
-import six.moves.urllib.parse as urlparse
+import queue
 import threading
-
-from ceilometermiddleware import ksa_adapter
+import urllib.parse as urlparse
 
 LOG = logging.getLogger(__name__)
 
@@ -131,6 +129,11 @@ class InputProxy(object):
         self.bytes_received += len(line)
         return line
 
+    def close(self):
+        close_method = getattr(self.wsgi_input, 'close', None)
+        if callable(close_method):
+            close_method()
+
 
 class KeystoneClientLoader(ksa_adapter.Adapter):
     """Keystone client adapter loader.
@@ -157,10 +160,25 @@ class Swift(object):
 
         self.ignore_projects = self._get_ignore_projects(conf)
 
+        extra_config_files = conf.get('extra_config_files')
+        if extra_config_files is not None:
+            extra_config_files = list_from_csv(extra_config_files)
+
+        extra_config_dirs = conf.get('extra_config_dirs')
+        if extra_config_dirs is not None:
+            extra_config_dirs = list_from_csv(extra_config_dirs)
+
+        oslo_conf = cfg.ConfigOpts()
+        oslo_conf([], project='swift',
+                  default_config_files=extra_config_files,
+                  default_config_dirs=extra_config_dirs,
+                  validate_default_values=True)
+
         oslo_messaging.set_transport_defaults(conf.get('control_exchange',
                                                        'swift'))
         self._notifier = oslo_messaging.Notifier(
-            oslo_messaging.get_transport(cfg.CONF, url=conf.get('url')),
+            oslo_messaging.get_notification_transport(oslo_conf,
+                                                      url=conf.get('url')),
             publisher_id='ceilometermiddleware',
             driver=conf.get('driver', 'messagingv2'),
             topics=[conf.get('topic', 'notifications')])
@@ -235,7 +253,7 @@ class Swift(object):
             return [client.projects.get(name_or_id)]
         except ksa_exc.NotFound:
             pass
-        if isinstance(name_or_id, six.binary_type):
+        if isinstance(name_or_id, bytes):
             name_or_id = name_or_id.decode('utf-8', 'strict')
         projects = client.projects.list(name=name_or_id)
         if not projects:
@@ -266,8 +284,14 @@ class Swift(object):
                 while chunk:
                     bytes_sent += len(chunk)
                     yield chunk
-                    chunk = next(iterator)
+                    try:
+                        chunk = next(iterator)
+                    except StopIteration:
+                        chunk = ''
             finally:
+                close_method = getattr(iterable, 'close', None)
+                if callable(close_method):
+                    close_method()
                 self.emit_event(env, input_proxy.bytes_received, bytes_sent)
 
         try:
@@ -280,20 +304,21 @@ class Swift(object):
 
     @_log_and_ignore_error
     def emit_event(self, env, bytes_received, bytes_sent, outcome='success'):
-        if ((env.get('HTTP_X_SERVICE_PROJECT_ID') or
-                env.get('HTTP_X_PROJECT_ID') or
-                env.get('HTTP_X_TENANT_ID')) in self.ignore_projects or
-                env.get('swift.source') is not None):
+        if (
+                (env.get('HTTP_X_SERVICE_PROJECT_ID')
+                 or env.get('HTTP_X_PROJECT_ID')
+                 or env.get('HTTP_X_TENANT_ID')) in self.ignore_projects
+                or env.get('swift.source') is not None):
             return
 
-        path = urlparse.quote(env['PATH_INFO'])
+        path = urlparse.quote(env.get('swift.backend_path', env['PATH_INFO']))
         method = env['REQUEST_METHOD']
         headers = {}
         for header in env:
             if header.startswith('HTTP_') and env[header]:
                 key = header[5:]
-                if isinstance(env[header], six.text_type):
-                    headers[key] = six.text_type(env[header])
+                if isinstance(env[header], str):
+                    headers[key] = str(env[header])
                 else:
                     headers[key] = str(env[header])
 
@@ -343,8 +368,8 @@ class Swift(object):
         initiator = cadf_resource.Resource(
             typeURI='service/security/account/user',
             id=env.get('HTTP_X_USER_ID'))
-        initiator.project_id = (env.get('HTTP_X_PROJECT_ID') or
-                                env.get('HTTP_X_TENANT_ID'))
+        initiator.project_id = (env.get('HTTP_X_PROJECT_ID')
+                                or env.get('HTTP_X_TENANT_ID'))
 
         # build notification body
         event = cadf_event.Event(eventTime=now, outcome=outcome,
